"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = exports.options = void 0;
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const graphile_config_1 = require("graphile-config");
const cli_1 = require("graphile-config/cli");
const load_1 = require("graphile-config/load");
const util_1 = require("util");
const stripAnsi_js_1 = require("../../../stripAnsi.js");
const inspectOpts = {
    colors: true,
};
function options(yargs) {
    return yargs
        .example("$0 -C graphile.config.ts", "Print your graphile.config.ts")
        .option("config", {
        alias: "C",
        type: "string",
        description: "The path to the config file",
        normalize: true,
    });
    // TODO: add options for debugging things like: where did this option come
    // from, why are the plugins in this order, etc.
}
exports.options = options;
async function run(args) {
    const userPreset = await (0, load_1.loadConfig)(args.config);
    if (!userPreset) {
        console.error("Failed to load config, please check the file exists");
        process.exit(1);
    }
    const resolvedPreset = (0, graphile_config_1.resolvePresets)([userPreset]);
    console.log(printPlugins(resolvedPreset.plugins));
    for (const key of Object.keys(resolvedPreset)) {
        if (key === "plugins" || key === "extends" || key === "disablePlugins") {
            continue;
        }
        console.log();
        console.log(chalk_1.default.whiteBright.bold(key) + ":");
        const value = resolvedPreset[key];
        if (Array.isArray(value)) {
            console.log((0, util_1.inspect)(value, inspectOpts));
        }
        else if (typeof value === "object" && value !== null) {
            const keys = Object.keys(value);
            if (keys.length) {
                for (const key of keys) {
                    const val = value[key];
                    console.log(`  ${chalk_1.default.blueBright(key)}: ${printValue(val, 4)}`);
                }
            }
            else {
                console.log(`  ${chalk_1.default.gray("-empty-")}`);
            }
        }
        else {
            console.log((0, util_1.inspect)(value, inspectOpts));
        }
    }
}
exports.run = run;
function printPlugins(plugins) {
    if (!plugins || plugins.length === 0) {
        return "";
    }
    return `${chalk_1.default.whiteBright.bold("plugins")}:
${plugins.map((p) => printPlugin(p)).join("\n")}`;
}
function printPlugin(plugin) {
    const left = `  ${chalk_1.default.greenBright.bold(plugin.name)}${chalk_1.default.whiteBright("@")}${chalk_1.default.gray(plugin.version)}${plugin.description ? ": " : ""}`;
    const l = (0, stripAnsi_js_1.stripAnsi)(left).length;
    const MAX = 50;
    const SCREEN_WIDTH = (0, cli_1.getTerminalWidth)();
    const padL = Math.max(MAX, l) - l;
    const pad = " ".repeat(Math.max(0, padL));
    return `${left}${plugin.description
        ? `${pad}${chalk_1.default.dim(oneLine(plugin.description, SCREEN_WIDTH - pad.length - (0, stripAnsi_js_1.stripAnsi)(left).length))}`
        : ""}`;
}
function oneLine(str, max = 60, suffix = "...") {
    const single = str.replace(/[\r\n\t]+/g, " ");
    if (single.length > max - suffix.length) {
        return single.slice(0, max - suffix.length) + suffix;
    }
    else {
        return single;
    }
}
function printValue(value, indentation) {
    const indent = " ".repeat(indentation);
    return (0, util_1.inspect)(value, inspectOpts).replace(/\n/g, `\n${indent}`);
}
//# sourceMappingURL=cli.js.map